// flip-std.cpp : Implementation file for flip-std.h.

#include "flip-std.h"
#include "program.h"

#include <fstream>
#include <iostream>

static std::istream* in;
static std::ostream* out;

static int input_callback()
{
    return in->get();
}

static void output_callback(const char* str)
{
    *out << str;
    out->flush();
}

int flip_std(const char* pszSourceFile, const char* pszInputFile, const char* pszOutputFile)
{
    program prog;
    std::ifstream source(pszSourceFile);
    prog.load(source);
    source.close();
    
    std::ifstream input;
    if (pszInputFile)
    {
        input.open(pszInputFile);
        in = &input;
    }
    else
        in = &std::cin;
    prog.set_input_callback(input_callback);
    
    std::ofstream output;
    if (pszOutputFile)
    {
        output.open(pszOutputFile);
        out = &output;
    }
    else
        out = &std::cout;
    prog.set_output_callback(output_callback);
    
    int result;
    while ((result = prog.step()) == 0);
    
    switch (result)
    {
        case 1:
            result = 0;
            break;
        case 2:
            std::cerr << "All balls are trapped or have left the program." << std::endl;
            break;
        default:
        {
            program::ball_t error_ball = prog.get_error_ball();
            std::cerr << "Invalid interaction between ball (" << error_ball.dir << ") and object '"
                        << prog.code()[error_ball.y][error_ball.x] << "' ("
                        << (int)(unsigned char)prog.code()[error_ball.y][error_ball.x]
                        << ") at line " << error_ball.y+1 << ", column " << error_ball.x+1 << "."
                        << std::endl;
            break;
        }
    }
    
    return result;
}
